/*
*         Copyright (c), NXP Semiconductors Bangalore / India
*
*                     (C)NXP Semiconductors
*       All rights are reserved. Reproduction in whole or in part is
*      prohibited without the written consent of the copyright owner.
*  NXP reserves the right to make changes without notice at any time.
* NXP makes no warranty, expressed, implied or statutory, including but
* not limited to any implied warranty of merchantability or fitness for any
* particular purpose, or that the use will not infringe any third party patent,
* copyright or trademark. NXP must not be liable for any loss or damage
*                          arising from its use.
*/

/** \file
* Example Source for NfcrdlibEx1_BasicDiscoveryLoop that uses the Discovery loop implementation.
* Uses both POLL and LISTEN modes of discovery loop. Displays detected tag information(like UID, SAK, Product Type)
* and prints information when it gets activated as a target by an external Initiator/reader.
* Whenever multiple technologies are detected, example will select first detected technology to resolve.
* Example will activate device at index zero whenever multiple device is detected.
*
* Please refer Readme.txt file for Hardware Pin Configuration, Software Configuration and steps to build and
* execute the project which is present in the same project directory.
* $Author: Purnank G (ing05193) $
* $Revision: 6017 $ (v05.19.00)
* $Date: 2016-09-19 19:12:31 +0530 (Mon, 19 Sep 2016) $
*
* History:
* BK: Generated 12. Jun 2014
* PC: Generated 25. Nov 2012
*
*/

/**
* Reader Library Headers
*/
#include <phApp_Init.h>
#include <cards.h>
/* Local headers */
#include "NfcrdlibEx1_BasicDiscoveryLoop.h"


/*TODO: Massimo Incerti added to support TFT*/
#include "lcd.h"


/*******************************************************************************
**   Global Defines
*******************************************************************************/

phacDiscLoop_Sw_DataParams_t       * pDiscLoop;       /* Discovery loop component */

/*The below variables needs to be initialized according to example requirements by a customer */

uint8_t  sens_res[2]     = {0x04, 0x00};              /* ATQ bytes - needed for anti-collision */
uint8_t  nfc_id1[3]      = {0xA1, 0xA2, 0xA3};        /* user defined bytes of the UID (one is hardcoded) - needed for anti-collision */
uint8_t  sel_res         = 0x40;
uint8_t  nfc_id3         = 0xFA;                      /* NFC3 byte - required for anti-collision */
uint8_t  poll_res[18]    = {0x01, 0xFE, 0xB2, 0xB3, 0xB4, 0xB5,
                                   0xB6, 0xB7, 0xC0, 0xC1, 0xC2, 0xC3,
                                   0xC4, 0xC5, 0xC6, 0xC7, 0x23, 0x45 };

#ifdef PHOSAL_FREERTOS_STATIC_MEM_ALLOCATION
uint32_t aBasicDiscTaskBuffer[BASIC_DISC_DEMO_TASK_STACK];
#else /* uint32_t aBasicDiscTaskBuffer[BASIC_DISC_DEMO_TASK_STACK]; */
#define aBasicDiscTaskBuffer    NULL
#endif /* PHOSAL_FREERTOS_STATIC_MEM_ALLOCATION */

void * palTop;
uint8_t  baSnepAppBuffer[256];		// Buffer to point to NDEF messages

//Required NDEF TEXT MESSAGE Format for the code to work: "ID;FirstName SecondName" For ID=1 access granted For other IDs access denied
//NDEF TEXT MESSAGE : "1;Ali Khan"
uint8_t	NDEF_TEXT[]=  { 0xD1, //Record Header - TNF-> Well Defined NDEF
						0x01, // Record Type Len
						0x0D, // Payload Length
						0x54, //Type Text
						0x02, // Language Code Length
				  0x65, 0x6E, // Language: en (English)
						0x31, // ID==1
		0x3B, 0x41, 0x6C, 0x69, 0x20, 0x4B, 0x68, 0x61, 0x6E, }; // Ali Khan

//NDEF TEXT MESSAGE : "0;John Doe"
uint8_t	NDEF_TEXT2[]=  { 0xD1, //Record Header - TNF-> Well Defined NDEF
						0x01, // Record Type Len
						0x0D, // Payload Length
						0x54, //Type Text
						0x02, // Language Code Length
				  0x65, 0x6E, // Language: en (English)
						0x30, // ID==0
						0x3B, 0x4A, 0x6F, 0x68, 0x6E, 0x20, 0x44, 0x6F, 0x65,}; // ;John Doe






/*******************************************************************************
**   Static Defines
*******************************************************************************/

/* This is used to save restore Poll Config.
 * If in case application has update/change PollCfg to resolve Tech
 * when Multiple Tech was detected in previous poll cycle
 */
static uint16_t bSavePollTechCfg;
static bool Write_Flag;
static bool button_press=0;
static bool print_flag=0;
/*******************************************************************************
**   Function Declarations
*******************************************************************************/
void BasicDiscoveryLoop_Demo(void  *pDataParams);
/* Reads whole NDEF Message and prints it. Needs to be called after card is detected and activated.
 *
 */
phStatus_t ReadNdefMessage(uint8_t TopTagType);


/* Demo Function to check Access from Well Defined NDEF MESSAGE (TEXT RECORD)
 * NDEF TEXT Message should have the following format for this to work: "ID;Name"
 */
void check_access(uint8_t *pBuffer,
        uint32_t dwBufferLength);

/* Interrupt callback function for wakeup switch
 * Toggling wake up switch enables or disables Writing predefined NDEF_TEXT to activated card
 */
void pint_intr_write_mode()
{

	button_press=1;
	if(Write_Flag)
	{
		DEBUG_PRINTF("Out of Write Mode\n");
		Write_Flag=0;
		return;

	}
	else
	{	DEBUG_PRINTF("Into Write Mode\n");
		DEBUG_PRINTF("Press Button Again to go out of Write Mode\n");
		Write_Flag=1;
		return;
	}
}
/* Prints welcome message on LCD
 */
void print_welcome_lcd();
/*******************************************************************************
**   Function Definitions
*******************************************************************************/


/*******************************************************************************
**   Main Function
*******************************************************************************/

int main (void)
{

	 /*TODO: Massimo Incerti adding TFT specific functionalities
	   * Had to add:
	   * fsl_spi_dma.c
	   * fls_spi_dma.h
	   * fsl_dma.c
	   * fsl_dma.h
	   * #include "lcd.h"*/

	/*TODO: MASSIMO INCERTI*/
	/*Moved init functions here at this stage*/


	/* Perform Controller specific initialization. */
	phApp_CPU_Init();

	/* Perform OSAL Initialization. */
	(void)phOsal_Init();

	DEBUG_PRINTF("\n\r\t*****Access Demo*****\n\r");

    do
    {
        phStatus_t            status = PH_ERR_INTERNAL_ERROR;
        phNfcLib_Status_t     dwStatus;

#ifdef PH_PLATFORM_HAS_ICFRONTEND
        phNfcLib_AppContext_t AppContext = {0};
#endif /* PH_PLATFORM_HAS_ICFRONTEND */

#ifndef PH_OSAL_NULLOS
        phOsal_ThreadObj_t BasicDisc;
#endif /* PH_OSAL_NULLOS */




/* TODO: MASSIMO INCERTI
 * MCU sprecific init originally here*/

    	/* Perform Controller specific initialization. */
    	//        phApp_CPU_Init();

    	        /* Perform OSAL Initialization. */
    	//        (void)phOsal_Init();

    	//        DEBUG_PRINTF("\n\r\t*****Access Demo*****\n\r");





#ifdef PH_PLATFORM_HAS_ICFRONTEND
        status = phbalReg_Init(&sBalParams, sizeof(phbalReg_Type_t));
        CHECK_STATUS(status);

        AppContext.pBalDataparams = &sBalParams;
        dwStatus = phNfcLib_SetContext(&AppContext);
        CHECK_NFCLIB_STATUS(dwStatus);
#endif


        phDriver_TimerStart(PH_DRIVER_TIMER_MILLI_SECS, 1000u, NULL); // Test Delay to solve ble nfc problem
        /* Initialize library */
        dwStatus = phNfcLib_Init();
        CHECK_NFCLIB_STATUS(dwStatus);
        if(dwStatus != PH_NFCLIB_STATUS_SUCCESS) break;

        /* Set the generic pointer */
        pHal = phNfcLib_GetDataParams(PH_COMP_HAL);
        pDiscLoop = phNfcLib_GetDataParams(PH_COMP_AC_DISCLOOP);
        palTop = phNfcLib_GetDataParams(PH_COMP_AL_TOP);


        /*Calling lcd_init*/

        lcd_init();
        PRINTF("\r\nLCD driver init done .\r\n");

        print_welcome_lcd(); // Prints to LCD Welcome message

        /* Initialize other components that are not initialized by NFCLIB and configure Discovery Loop. */
        status = phApp_Comp_Init(pDiscLoop);
        CHECK_STATUS(status);
        if(status != PH_ERR_SUCCESS) break;

        /* Configure the IRQ */
        status = phApp_Configure_IRQ();
        CHECK_STATUS(status);
        if(status != PH_ERR_SUCCESS) break;

        //Setting up User interrupt to go into card write mode
        /* Setup Pin Interrupt 1 for falling edge */
        PINT_PinInterruptConfig(PINT, kPINT_PinInt1, kPINT_PinIntEnableRiseEdge, pint_intr_write_mode);
        /* Enable callbacks for PINT1 by Index */
        PINT_EnableCallbackByIndex(PINT, kPINT_PinInt1);

#ifndef PH_OSAL_NULLOS

        BasicDisc.pTaskName = (uint8_t *) "BasicDiscLoop";
        BasicDisc.pStackBuffer = aBasicDiscTaskBuffer;
        BasicDisc.priority = BASIC_DISC_DEMO_TASK_PRIO;
        BasicDisc.stackSizeInNum = BASIC_DISC_DEMO_TASK_STACK;
        phOsal_ThreadCreate(&BasicDisc.ThreadHandle, &BasicDisc, &BasicDiscoveryLoop_Demo, pDiscLoop);
        phOsal_StartScheduler();
        DEBUG_PRINTF("RTOS Error : Scheduler exited. \n\r");

#else
        (void)BasicDiscoveryLoop_Demo(pDiscLoop);
#endif
    } while(0);

    while(1); //Comes here if initialization failure or scheduler exit due to error

    return 0;
}

/**
* This function demonstrates the usage of discovery loop
* It detects and reports the NFC technology type
* \param   pDataParams      The discovery loop data parameters
* \note    This function will never return
*/
void BasicDiscoveryLoop_Demo(void  *pDataParams)
{
    phStatus_t    status, statustmp;
    uint16_t      wTagsDetected = 0;
    uint16_t      wNumberOfTags = 0;
    uint16_t      wEntryPoint;
    uint16_t      wValue;
    uint8_t       bIndex;


    status = phApp_HALConfigAutoColl();
    CHECK_STATUS(status);

    /* Get Poll Configuration */
    status = phacDiscLoop_GetConfig(pDataParams, PHAC_DISCLOOP_CONFIG_PAS_POLL_TECH_CFG, &bSavePollTechCfg);
    CHECK_STATUS(status);

    /* Start in poll mode */
    wEntryPoint = PHAC_DISCLOOP_ENTRY_POINT_POLL;
    status = PHAC_DISCLOOP_LPCD_NO_TECH_DETECTED;

    while(1)
    {
    	if(button_press)
    	{
    		if(Write_Flag)
    		{
    			lcd_clear_screen(LCD_COLOR_MAGENTA);
    			lcd_display_string(40,150, (uint8_t *)"Write Mode ON", 16, LCD_COLOR_BLACK);

    			phDriver_TimerStart(PH_DRIVER_TIMER_MILLI_SECS, 500u, NULL); // Add some delay before clearing the screen again
    			print_welcome_lcd();

    		}
    		else
    		{
    			lcd_clear_screen(LCD_COLOR_MAGENTA);
    			lcd_display_string(40,150, (uint8_t *)"Write Mode OFF", 16, LCD_COLOR_BLACK);
    			phDriver_TimerStart(PH_DRIVER_TIMER_MILLI_SECS, 500u, NULL); // Add some delay before clearing the screen again
    			print_welcome_lcd();
    		}
    		button_press=0;

    	}

        /* Switch off RF field */
        statustmp = phhalHw_FieldOff(pHal);
        CHECK_STATUS(statustmp);

        /* Set Discovery Poll State to Detection */
        statustmp = phacDiscLoop_SetConfig(pDataParams, PHAC_DISCLOOP_CONFIG_NEXT_POLL_STATE, PHAC_DISCLOOP_POLL_STATE_DETECTION);
        CHECK_STATUS(statustmp);

        /* Set Poll Configuration */
        statustmp = phacDiscLoop_SetConfig(pDataParams, PHAC_DISCLOOP_CONFIG_PAS_POLL_TECH_CFG, bSavePollTechCfg);
        CHECK_STATUS(statustmp);

#ifdef PH_EXAMPLE1_LPCD_ENABLE

#ifdef NXPBUILD__PHHAL_HW_RC663
        if (wEntryPoint == PHAC_DISCLOOP_ENTRY_POINT_POLL)
#else
        /* Configure LPCD */
        if ((status & PH_ERR_MASK) == PHAC_DISCLOOP_LPCD_NO_TECH_DETECTED)
#endif
        {
            status = phApp_ConfigureLPCD();
            CHECK_STATUS(status);
        }

        /* Bool to enable LPCD feature. */
        status = phacDiscLoop_SetConfig(pDataParams, PHAC_DISCLOOP_CONFIG_ENABLE_LPCD, PH_ON);
        CHECK_STATUS(status);
#endif /* PH_EXAMPLE1_LPCD_ENABLE */

        /* Start discovery loop */
        status = phacDiscLoop_Run(pDataParams, wEntryPoint);

        if(wEntryPoint == PHAC_DISCLOOP_ENTRY_POINT_POLL)
        {
            if((status & PH_ERR_MASK) == PHAC_DISCLOOP_MULTI_TECH_DETECTED)
            {
                DEBUG_PRINTF (" \n\r Multiple technology detected: \n\r");

                status = phacDiscLoop_GetConfig(pDataParams, PHAC_DISCLOOP_CONFIG_TECH_DETECTED, &wTagsDetected);
                CHECK_STATUS(status);

                if(PHAC_DISCLOOP_CHECK_ANDMASK(wTagsDetected, PHAC_DISCLOOP_POS_BIT_MASK_A))
                {
                    DEBUG_PRINTF (" \tType A detected... \n\r");
                }
                if(PHAC_DISCLOOP_CHECK_ANDMASK(wTagsDetected, PHAC_DISCLOOP_POS_BIT_MASK_B))
                {
                    DEBUG_PRINTF (" \tType B detected... \n\r");
                }
                if(PHAC_DISCLOOP_CHECK_ANDMASK(wTagsDetected, PHAC_DISCLOOP_POS_BIT_MASK_F212))
                {
                    DEBUG_PRINTF (" \tType F detected with baud rate 212... \n\r");
                }
                if(PHAC_DISCLOOP_CHECK_ANDMASK(wTagsDetected, PHAC_DISCLOOP_POS_BIT_MASK_F424))
                {
                    DEBUG_PRINTF (" \tType F detected with baud rate 424... \n\r");
                }
                if(PHAC_DISCLOOP_CHECK_ANDMASK(wTagsDetected, PHAC_DISCLOOP_POS_BIT_MASK_V))
                {
                    DEBUG_PRINTF(" \tType V / ISO 15693 / T5T detected... \n\r");
                }

                /* Select 1st Detected Technology to Resolve*/
                for(bIndex = 0; bIndex < PHAC_DISCLOOP_PASS_POLL_MAX_TECHS_SUPPORTED; bIndex++)
                {
                    if(PHAC_DISCLOOP_CHECK_ANDMASK(wTagsDetected, (1 << bIndex)))
                    {
                        /* Configure for one of the detected technology */
                        status = phacDiscLoop_SetConfig(pDataParams, PHAC_DISCLOOP_CONFIG_PAS_POLL_TECH_CFG, (1 << bIndex));
                        CHECK_STATUS(status);
                        break;
                    }
                }

                /* Print the technology resolved */
                phApp_PrintTech((1 << bIndex));

                /* Set Discovery Poll State to collision resolution */
                status = phacDiscLoop_SetConfig(pDataParams, PHAC_DISCLOOP_CONFIG_NEXT_POLL_STATE, PHAC_DISCLOOP_POLL_STATE_COLLISION_RESOLUTION);
                CHECK_STATUS(status);

                /* Restart discovery loop in poll mode from collision resolution phase */
                status = phacDiscLoop_Run(pDataParams, wEntryPoint);
            }

            if((status & PH_ERR_MASK) == PHAC_DISCLOOP_MULTI_DEVICES_RESOLVED)
            {
                /* Get Detected Technology Type */
                status = phacDiscLoop_GetConfig(pDiscLoop, PHAC_DISCLOOP_CONFIG_TECH_DETECTED, &wTagsDetected);
                CHECK_STATUS(status);

                /* Get number of tags detected */
                status = phacDiscLoop_GetConfig(pDiscLoop, PHAC_DISCLOOP_CONFIG_NR_TAGS_FOUND, &wNumberOfTags);
                CHECK_STATUS(status);

                DEBUG_PRINTF (" \n\r Multiple cards resolved: %d cards \n\r",wNumberOfTags);
                phApp_PrintTagInfo(pDataParams, wNumberOfTags, wTagsDetected);

                if(wNumberOfTags > 1)
                {
                    /* Get 1st Detected Tag and Activate device at index 0 */
                    for(bIndex = 0; bIndex < PHAC_DISCLOOP_PASS_POLL_MAX_TECHS_SUPPORTED; bIndex++)
                    {
                        if(PHAC_DISCLOOP_CHECK_ANDMASK(wTagsDetected, (1 << bIndex)))
                        {
                            DEBUG_PRINTF("\t Activating one card...\n\r");
                            status = phacDiscLoop_ActivateCard(pDataParams, bIndex, 0);
                            break;
                        }
                    }

                    if(((status & PH_ERR_MASK) == PHAC_DISCLOOP_DEVICE_ACTIVATED) ||
                            ((status & PH_ERR_MASK) == PHAC_DISCLOOP_PASSIVE_TARGET_ACTIVATED) ||
                            ((status & PH_ERR_MASK) == PHAC_DISCLOOP_MERGED_SEL_RES_FOUND))
                    {
                        /* Get Detected Technology Type */
                        status = phacDiscLoop_GetConfig(pDiscLoop, PHAC_DISCLOOP_CONFIG_TECH_DETECTED, &wTagsDetected);
                        CHECK_STATUS(status);

                        phApp_PrintTagInfo(pDataParams, 0x01, wTagsDetected);
                    }
                    else
                    {
                        PRINT_INFO("\t\tCard activation failed...\n\r");
                    }
                }
                /* Switch to LISTEN mode after POLL mode */
            }
            else if (((status & PH_ERR_MASK) == PHAC_DISCLOOP_NO_TECH_DETECTED) ||
                    ((status & PH_ERR_MASK) == PHAC_DISCLOOP_NO_DEVICE_RESOLVED))
            {
            	/* Switch to LISTEN mode after POLL mode */
            	if(print_flag)
            	{
            		phDriver_TimerStart(PH_DRIVER_TIMER_MILLI_SECS, 1000u, NULL); // Add some delay before clearing the screen again
            		print_welcome_lcd();
            		print_flag=0;
            	}

            }
            else if((status & PH_ERR_MASK) == PHAC_DISCLOOP_EXTERNAL_RFON)
            {
                /*
                 * If external RF is detected during POLL, return back so that the application
                 * can restart the loop in LISTEN mode
                 */
            }
            else if((status & PH_ERR_MASK) == PHAC_DISCLOOP_MERGED_SEL_RES_FOUND)
            {
                DEBUG_PRINTF (" \n\r Device having T4T and NFC-DEP support detected... \n\r");

                /* Get Detected Technology Type */
                status = phacDiscLoop_GetConfig(pDiscLoop, PHAC_DISCLOOP_CONFIG_TECH_DETECTED, &wTagsDetected);
                CHECK_STATUS(status);

                phApp_PrintTagInfo(pDataParams, 1, wTagsDetected);

                /* Switch to LISTEN mode after POLL mode */
            }
            else if((status & PH_ERR_MASK) == PHAC_DISCLOOP_DEVICE_ACTIVATED)
            {
                DEBUG_PRINTF (" \n\r Card detected and activated successfully... \n\r");
                status = phacDiscLoop_GetConfig(pDataParams, PHAC_DISCLOOP_CONFIG_NR_TAGS_FOUND, &wNumberOfTags);
                CHECK_STATUS(status);

                /* Get Detected Technology Type */
                status = phacDiscLoop_GetConfig(pDataParams, PHAC_DISCLOOP_CONFIG_TECH_DETECTED, &wTagsDetected);
                CHECK_STATUS(status);

                phApp_PrintTagInfo(pDataParams, wNumberOfTags, wTagsDetected);


               // READ NDEF
               uint8_t bTagState;   // variable used to represent state of tag

               status = ReadNdefMessage(PHAL_TOP_TAG_TYPE_T4T_TAG);
               if (status != PH_ERR_SUCCESS)
               	   {
            	   	   DEBUG_ERROR_PRINT(status);
                   }

               // Writes to card if the wake up button has been pressed
                if(Write_Flag)
               {
            	   PRINTF ("\n Writing Tag: \n");
            	   // for NDEF Write
            	   if(phalTop_CheckNdef(palTop,&bTagState)==PHAL_TOP_ERR_NON_NDEF_TAG) // checks if tag is not formatted for NDEF
            	   {
            		   status = phalTop_FormatNdef(palTop);
            	   }
            	   status=phalTop_WriteNdef(palTop,NDEF_TEXT,(sizeof(NDEF_TEXT)/sizeof(uint8_t)));
            		print_flag=0;// Print access status again
               }





                /* Switch to LISTEN mode after POLL mode */
            }
            else if((status & PH_ERR_MASK) == PHAC_DISCLOOP_ACTIVE_TARGET_ACTIVATED)
            {
                DEBUG_PRINTF (" \n\r Active target detected... \n\r");
                /* Switch to LISTEN mode after POLL mode */
            }
            else if((status & PH_ERR_MASK) == PHAC_DISCLOOP_PASSIVE_TARGET_ACTIVATED)
            {
                DEBUG_PRINTF (" \n\r Passive target detected... \n\r");

                /* Get Detected Technology Type */
                status = phacDiscLoop_GetConfig(pDiscLoop, PHAC_DISCLOOP_CONFIG_TECH_DETECTED, &wTagsDetected);
                CHECK_STATUS(status);

                phApp_PrintTagInfo(pDataParams, 1, wTagsDetected);

                /* Switch to LISTEN mode after POLL mode */
            }
            else if ((status & PH_ERR_MASK) == PHAC_DISCLOOP_LPCD_NO_TECH_DETECTED)
            {
                /* LPCD is succeed but no tag is detected. */
            }
            else
            {
                if((status & PH_ERR_MASK) == PHAC_DISCLOOP_FAILURE)
                {
                    status = phacDiscLoop_GetConfig(pDataParams, PHAC_DISCLOOP_CONFIG_ADDITIONAL_INFO, &wValue);
                    CHECK_STATUS(status);
                    DEBUG_ERROR_PRINT(PrintErrorInfo(wValue));
                }
                else
                {
                    DEBUG_ERROR_PRINT(PrintErrorInfo(status));
                }
            }

            /* Update the Entry point to LISTEN mode. */
            wEntryPoint = PHAC_DISCLOOP_ENTRY_POINT_LISTEN;
        }
        else
        {
            if((status & PH_ERR_MASK) == PHAC_DISCLOOP_EXTERNAL_RFOFF)
            {
                /*
                 * Enters here if in the target/card mode and external RF is not available
                 * Wait for LISTEN timeout till an external RF is detected.
                 * Application may choose to go into standby at this point.
                 */
                status = phhalHw_EventConsume(pHal);
                CHECK_STATUS(status);

                status = phhalHw_SetConfig(pHal, PHHAL_HW_CONFIG_RFON_INTERRUPT, PH_ON);
                CHECK_STATUS(status);

                status = phhalHw_EventWait(pHal, LISTEN_PHASE_TIME_MS);
                if((status & PH_ERR_MASK) == PH_ERR_IO_TIMEOUT)
                {
                    wEntryPoint = PHAC_DISCLOOP_ENTRY_POINT_POLL;
                }
                else
                {
                    wEntryPoint = PHAC_DISCLOOP_ENTRY_POINT_LISTEN;
                }
            }
            else
            {
                if((status & PH_ERR_MASK) == PHAC_DISCLOOP_ACTIVATED_BY_PEER)
                {
                    DEBUG_PRINTF (" \n\r Device activated in listen mode... \n\r");
                }
                else if ((status & PH_ERR_MASK) == PH_ERR_INVALID_PARAMETER)
                {
                    /* In case of Front end used is RC663, then listen mode is not supported.
                     * Switch from listen mode to poll mode. */
                }
                else
                {
                    if((status & PH_ERR_MASK) == PHAC_DISCLOOP_FAILURE)
                    {
                        status = phacDiscLoop_GetConfig(pDataParams, PHAC_DISCLOOP_CONFIG_ADDITIONAL_INFO, &wValue);
                        CHECK_STATUS(status);
                        DEBUG_ERROR_PRINT(PrintErrorInfo(wValue));
                    }
                    else
                    {
                        DEBUG_ERROR_PRINT(PrintErrorInfo(status));
                    }
                }

                /* On successful activated by Peer, switch to LISTEN mode */
                wEntryPoint = PHAC_DISCLOOP_ENTRY_POINT_POLL;
            }
        }
    }
}


/**
* Reads NDEF Message if the detected tag has a valid NDEF message
*/
phStatus_t ReadNdefMessage(uint8_t TopTagType)
{
    phStatus_t status;
    uint8_t bTagState;

uint16_t wDataLength = 0;

    /* Configure Top layer for specified tag type */
    status = phalTop_SetConfig(palTop, PHAL_TOP_CONFIG_TAG_TYPE, TopTagType);
    DEBUG_ERROR_PRINT(status);

    /* Check for NDEF presence */
    status = phalTop_CheckNdef(palTop, &bTagState);
    DEBUG_ERROR_PRINT(status);

    if((bTagState == PHAL_TOP_STATE_READONLY) || (bTagState == PHAL_TOP_STATE_READWRITE))
    {
        /* Read NDEF message */
        status = phalTop_ReadNdef(palTop, baSnepAppBuffer, &wDataLength);
        DEBUG_ERROR_PRINT(status);

        /* Print NDEF message, if not NULL NDEF */
        if(wDataLength)
        {
           // PRINTF("\tNDEF detected...\n");
           // PRINTF("\tNDEF length: %d\n", wDataLength);
            //PRINTF("\t\n\nNDEF message:\n");
            //DumpBuffer(aData, wDataLength);
            DumpBuffer(baSnepAppBuffer, wDataLength);  //Uncomment this to print the ndef message
            check_access(baSnepAppBuffer,wDataLength);
        }
        else
        {
            PRINTF("\tNDEF content is NULL...\n");
            if(!print_flag)
            {
            	lcd_clear_screen(LCD_COLOR_RED);
            	lcd_display_string(40,100, (uint8_t *)"*Access Denied*", 16, LCD_COLOR_WHITE);
            	lcd_display_string(40,150, (uint8_t *)"NDEF content NULL", 16, LCD_COLOR_WHITE);
            	print_flag=1;
            }
        }
    }
    else
    {
        PRINTF("\tNo NDEF content detected...\n");
        if(!print_flag)
        {
        	lcd_clear_screen(LCD_COLOR_RED);
        	lcd_display_string(40,100, (uint8_t *)"*Access Denied*", 16, LCD_COLOR_WHITE);
        	lcd_display_string(40,150, (uint8_t *)"No NDEF content", 16, LCD_COLOR_WHITE);
        	print_flag=1;
        }
    }

    return status;
}

void check_access(uint8_t *pBuffer,
        uint32_t NdefLength)

{
	uint8_t ID;
	char *name;
	ID=pBuffer[7]; // ID is the 8th byte of NDEF Text Message - see NDEF_TEXT

	name = &pBuffer[9];
	name[NdefLength-9]='\0';

	if(ID=='1')
	{
		PRINTF("\n\n\t****Access Granted****\n\n"); /// PRINT TO LCD SCREEN ACCESS GRANTED and name of user can be taken from name
		PRINTF("\t\tWelcome: ");
		PRINTF(name);
		PRINTF("\n");
		if(!print_flag)
		{
			lcd_clear_screen(LCD_COLOR_GREEN);
			lcd_display_string(40,100, (uint8_t *)"*Access Granted*", 16, LCD_COLOR_BLACK);
			lcd_display_string(40,150, (uint8_t *)"Welcome: ", 16, LCD_COLOR_BLACK);
			lcd_display_string(40,200, name, 16, LCD_COLOR_BLACK);
			print_flag=1;
		}

	}
	else
	{
		PRINTF("\n\n\t****Access Denied****\n\n");			/// PRINT TO LCD ACCESS Denied
		PRINTF("\tID %c not registered\n",ID);
		PRINTF("\tSorry: ");
		PRINTF(name);
		PRINTF("\n\n\n");
		if(!print_flag)
		{
			lcd_clear_screen(LCD_COLOR_RED);
			lcd_display_string(40,100, (uint8_t *)"*Access Denied*", 16, LCD_COLOR_WHITE);
			lcd_display_string(40,150, (uint8_t *)"Sorry ", 16, LCD_COLOR_WHITE);
			lcd_display_string(40,200, name, 16, LCD_COLOR_WHITE);
			print_flag=1;
		}

	}



}

void print_welcome_lcd()
{
	lcd_clear_screen(LCD_COLOR_BLUE);
	lcd_display_string(40,50, (uint8_t *)"WELCOME TO NXP", 16, LCD_COLOR_WHITE);
	lcd_display_string(40,100, (uint8_t *)"LPC5500", 16, LCD_COLOR_WHITE);
	lcd_display_string(40,150, (uint8_t *)"Authentication demo", 16, LCD_COLOR_WHITE);
	lcd_display_string(40,200, (uint8_t *)"EMBEDDED WORLD", 16, LCD_COLOR_WHITE);
	lcd_display_string(40,250, (uint8_t *)"2019", 16, LCD_COLOR_WHITE);
}

/******************************************************************************
**                            End Of File
******************************************************************************/
